# Copyright 2013 New Dream Network, LLC (DreamHost)
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import re
import six
from six.moves.urllib import parse
import string
import subprocess


CGIT_BASE = 'http://git.openstack.org/cgit/'


def _guess_cgit_link():
    try:
        git_remote = subprocess.check_output(
            ['git', 'config', '--local', '--get', 'remote.origin.url']
        )
    except (OSError, subprocess.CalledProcessError):
        # git is not present or the command failed
        return None
    else:
        if six.PY3:
            git_remote = os.fsdecode(git_remote)
        parsed = parse.urlparse(git_remote)
        parsed = '/'.join(parsed.path.rstrip('/').split('/')[-2:])
        parsed = re.sub(r'\.git$', '', parsed)
        return CGIT_BASE + parsed


def _html_page_context(app, pagename, templatename, context, doctree):
    # Insert the cgit link into the template context.
    context['cgit_link'] = app.config.oslosphinx_cgit_link

    git_cmd = ["git", "tag"]
    try:
        raw_version_list = subprocess.Popen(
            git_cmd, stdout=subprocess.PIPE).communicate()[0]
    except OSError:
        app.warn('Cannot get tags from git repository. '
                 'Not setting "other_versions".')
        raw_version_list = ''

    # grab last five that start with a number and reverse the order
    if six.PY3:
        raw_version_list = raw_version_list.decode("utf8")
    other_versions = [t for t in raw_version_list.split('\n')
                      if t and t[0] in string.digits][:-6:-1]
    context['other_versions'] = other_versions
    return None


def builder_inited(app):
    theme_dir = os.path.join(os.path.dirname(__file__), 'theme')
    app.info('Using openstack theme from %s' % theme_dir)
    # Insert our theme directory at the front of the search path and
    # force the theme setting to use the one in the package unless
    # another openstack theme is already selected. This is done here,
    # instead of in setup(), because conf.py is read after setup()
    # runs, so if the conf contains these values the user values
    # overwrite these. That's not bad for the theme, but it breaks the
    # search path.
    app.config.html_theme_path.insert(0, theme_dir)
    # Set the theme name
    if not app.config.html_theme.startswith('openstack'):
        app.config.html_theme = 'openstack'
    # Re-initialize the builder, if it has the method for setting up
    # the templates and theme.
    if hasattr(app.builder, 'init_templates'):
        app.builder.init_templates()
    # Register our page context additions
    app.connect('html-page-context', _html_page_context)


def setup(app):
    app.connect('builder-inited', builder_inited)
    # Try to guess at the default value for where the cgit repository
    # is.
    cgit_link = _guess_cgit_link()
    app.add_config_value('oslosphinx_cgit_link', cgit_link, 'env')



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Ensure that the name of the spec file matches the name of a blueprint.
"""

import requests


class BlueprintChecker(object):

    def __init__(self, app):
        self.app = app
        self.project_names = []
        self._good_bps = set()
        self._prefix = None
        self._warn_search = 'unset'

    BP_URL_TEMPLATE = 'https://api.launchpad.net/devel/%s/+spec/%s'
    PROJ_LIST_URL_TEMPLATE = 'https://api.launchpad.net/1.0/%s/projects'

    def _load_project_settings(self):
        if self.project_names:
            return
        # If a project_name is set in the configuration, use
        # that. Otherwise, allow any project in the project group.
        project_name = self.app.config.check_blueprints_project
        pg_name = self.app.config.check_blueprints_project_group
        if project_name:
            self.project_names = [project_name]
            self._warn_search = 'the %s project' % project_name
        else:
            proj_list_response = requests.get(self.PROJ_LIST_URL_TEMPLATE
                                              % pg_name)
            projects = proj_list_response.json()['entries']
            self.project_names = [p['name'] for p in projects]
            self._warn_search = ('any projects in the %s project group'
                                 % pg_name)

    @property
    def desired_prefix(self):
        """Determine the prefix for files we care to check.

        We only care about blueprints in the current release, if the
        check_blueprints_release option is set.

        """
        if self._prefix is None:
            release = self.app.config.check_blueprints_release
            if release:
                self._prefix = 'specs/%s/' % release
            else:
                self._prefix = 'specs/'
        return self._prefix

    def doctree_resolved(self, app, doctree, docname):
        """Hook registered as event handler."""
        if not docname.startswith(self.desired_prefix):
            return
        bp_name = docname.split('/')[-1]
        if bp_name == 'index':
            return
        self.check(bp_name)

    def blueprint_exists(self, project_name, bp_name):
        """Return boolean indicating whether the blueprint exists."""
        self.app.info('Checking for %s in %s' % (bp_name, project_name))
        url = self.BP_URL_TEMPLATE % (project_name, bp_name)
        response = requests.get(url)
        if response.status_code == 200:
            self.app.info('Found %s in %s' % (bp_name, project_name))
            return True
        return False

    def check(self, bp_name):
        """Given one blueprint name, check to see if it is valid."""
        if bp_name in self._good_bps:
            return True
        self._load_project_settings()
        self.app.info('')  # emit newline
        candidate_project, dash, bp_name_to_find = bp_name.partition('-')
        if candidate_project in self.project_names:
            # First check the shortened name of the blueprint in the project.
            if self.blueprint_exists(candidate_project, bp_name_to_find):
                return
            # Then check the full name of the blueprint in the project.
            if self.blueprint_exists(candidate_project, bp_name):
                return
            self.app.info(
                ('Blueprint name %r looks like it starts with a project '
                 'name, but %r was not found in project %r') %
                (bp_name, bp_name_to_find, candidate_project)
            )
        else:
            self.app.info(
                'Blueprint checking is faster if the file names '
                'start with the launchpad project name.'
            )
        for project_name in self.project_names:
            if self.blueprint_exists(project_name, bp_name):
                self._good_bps.add(bp_name)
                break
        else:
            self.app.warn(
                'Could not find a blueprint called %r in %s'
                % (bp_name, self._warn_search),
                location=(bp_name, 0),
            )
            raise ValueError(
                'Document %s does not match any blueprint name in %s'
                % (bp_name, self._warn_search))


def setup(app):
    app.info('Initializing %s' % __name__)
    checker = BlueprintChecker(app)
    app.connect('doctree-resolved', checker.doctree_resolved)
    app.add_config_value('check_blueprints_project_group', 'openstack', 'env')
    app.add_config_value('check_blueprints_project', '', 'env')
    app.add_config_value('check_blueprints_release', '', 'env')



# Copyright (c) 2013 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Intersphinx mapping file for the OpenStack projects.

To use this mapping in a project, first add 'sphinx.ext.intersphinx' to
your extensions list in conf.py. Then::

   from oslosphinx import intersphinx
   intersphinx_mapping = intersphinx.mapping

You'll be able to link to other project's documentation, such as:

    :ref:`virtual-environments <nova:virtual-environments>`
"""

mapping = dict(
    # OpenStack Projects
    ceilometer=('http://docs.openstack.org/developer/ceilometer/', None),
    cinder=('http://docs.openstack.org/developer/cinder/', None),
    glance=('http://docs.openstack.org/developer/glance/', None),
    heat=('http://docs.openstack.org/developer/heat/', None),
    horizon=('http://docs.openstack.org/developer/horizon/', None),
    ironic=('http://docs.openstack.org/developer/ironic/', None),
    keystone=('http://docs.openstack.org/developer/keystone/', None),
    nova=('http://docs.openstack.org/developer/nova/', None),
    oslo_config=('http://docs.openstack.org/developer/oslo.config/', None),
    oslo_messaging=(
        'http://docs.openstack.org/developer/oslo.messaging/', None),
    ceilometerclient=(
        'http://docs.openstack.org/developer/python-ceilometerclient/', None),
    cinderclient=(
        'http://docs.openstack.org/developer/python-cinderclient/', None),
    glanceclient=(
        'http://docs.openstack.org/developer/python-glanceclient/', None),
    heatclient=(
        'http://docs.openstack.org/developer/python-heatclient/', None),
    ironicclient=(
        'http://docs.openstack.org/developer/python-ironicclient/', None),
    keystoneclient=(
        'http://docs.openstack.org/developer/python-keystoneclient/', None),
    novaclient=(
        'http://docs.openstack.org/developer/python-novaclient/', None),
    openstackclient=(
        'http://docs.openstack.org/developer/python-openstackclient/', None),
    quantumclient=(
        'http://docs.openstack.org/developer/python-swiftclient/', None),
    swiftclient=(
        'http://docs.openstack.org/developer/python-swiftclient/', None),
    troveclient=(
        'http://docs.openstack.org/developer/python-troveclient/', None),
    quantum=('http://docs.openstack.org/developer/quantum/', None),
    swift=('http://docs.openstack.org/developer/swift/', None),
    trove=('http://docs.openstack.org/developer/trove/', None),
    # Other things of note
    python=('http://docs.python.org/', None),
    infra=('http://docs.openstack.org/infra/system-config/', None),
    zuul=('http://docs.openstack.org/infra/zuul/', None),
)
